@@ -7,13 +7,17 @@ Provides the two commands for preparing an application:
 
 import os
 import sys
+if sys.version_info < (2, 4):
+    from paste.script.util import string24 as string
+else:
+    import string
 import new
 from cStringIO import StringIO
 from paste.script.command import Command, BadCommand, run as run_command
 import paste.script.templates
 from paste.script import copydir
 import pkg_resources
-from Cheetah.Template import Template
+Cheetah = None
 from ConfigParser import ConfigParser
 from paste.util import import_string
 from paste.deploy import appconfig
@@ -443,6 +447,9 @@ class SetupCommand(AbstractInstallCommand):
         ep_name = conf.context.entry_point_name
         ep_group = conf.context.protocol
         dist = conf.context.distribution
+        if dist is None:
+            raise BadCommand(
+                "The section %r is not the application (probably a filter).  You should add #section_name, where section_name is the section that configures your application" % plain_section)
         installer = self.get_installer(dist, ep_group, ep_name)
         installer.setup_config(
             self, config_file, section, self.sysconfig_install_vars(installer))
@@ -478,6 +485,10 @@ class Installer(object):
     # specified:
     default_config_filename = None
 
+    # Set this to true to use Cheetah to fill your templates, or false
+    # to not do so:
+    use_cheetah = True
+
     def __init__(self, dist, ep_group, ep_name):
         self.dist = dist
         self.ep_group = ep_group
@@ -517,15 +528,21 @@ class Installer(object):
         ``Package.egg-info/paste_deploy_config.ini_tmpl`` and fills it
         with the variables.
         """
+        global Cheetah
         meta_name = 'paste_deploy_config.ini_tmpl'
         if not self.dist.has_metadata(meta_name):
             if command.verbose:
                 print 'No %s found' % meta_name
             return self.simple_config(vars)
-        tmpl = Template(self.dist.get_metadata(meta_name),
-                        searchList=[vars])
-        return copydir.careful_sub(
-            tmpl, vars, meta_name)
+        if self.use_cheetah:
+            import Cheetah.Template
+            tmpl = Cheetah.Template.Template(self.dist.get_metadata(meta_name),
+                                    searchList=[vars])
+            return copydir.careful_sub(
+                tmpl, vars, meta_name)
+        else:
+            tmpl = string.Template(self.dist.get_metadata(meta_name))
+            return tmpl.substitute(vars)
 
     def simple_config(self, vars):
         """
@@ -553,20 +570,17 @@ class Installer(object):
         the extra attributes ``global_conf``, ``local_conf`` and
         ``filename``
         """
-        for line in self.dist.get_metadata_lines('top_level.txt'):
-            line = line.strip()
-            if not line or line.startswith('#'):
-                continue
-            mod_name = line + '.websetup'
-            try:
-                mod = import_string.import_module(mod_name)
-            except ImportError:
-                if command.verbose > 1:
-                    import traceback
-                    print 'Error loading module %s for setup' % mod_name
-                    traceback.print_exc(file=sys.stdout)
-                elif command.verbose > 0:
-                    print 'No %s module found for setup' % mod_name
+        modules = [
+            line.strip()
+            for line in self.dist.get_metadata_lines('top_level.txt')
+            if line.strip() and not line.strip().startswith('#')]
+        if not modules:
+            print 'No modules are listed in top_level.txt'
+            print 'Try running python setup.py egg_info to regenerate that file'
+        for mod_name in modules:
+            mod_name = mod_name + '.websetup'
+            mod = import_string.try_import_module(mod_name)
+            if mod is None:
                 continue
             if command.verbose:
                 print 'Running setup_config() from %s' % mod_name
