@@ -16,9 +16,50 @@ except ImportError:
     from paste.script.util import subprocess24 as subprocess
 import inspect
 
+class SkipTemplate(Exception):
+    """
+    Raised to indicate that the template should not be copied over.
+    Raise this exception during the substitution of your template
+    """
+
 def copy_dir(source, dest, vars, verbosity, simulate, indent=0,
              use_cheetah=False, sub_vars=True, interactive=False,
              svn_add=True, overwrite=True, template_renderer=None):
+    """
+    Copies the ``source`` directory to the ``dest`` directory.
+
+    ``vars``: A dictionary of variables to use in any substitutions.
+
+    ``verbosity``: Higher numbers will show more about what is happening.
+
+    ``simulate``: If true, then don't actually *do* anything.
+
+    ``indent``: Indent any messages by this amount.
+
+    ``sub_vars``: If true, variables in ``_tmpl`` files and ``+var+``
+    in filenames will be substituted.
+
+    ``use_cheetah``: If true, then any templates encountered will be
+    substituted with Cheetah.  Otherwise ``template_renderer`` or
+    ``string.Template`` will be used for templates.
+
+    ``svn_add``: If true, any files written out in directories with
+    ``.svn/`` directories will be added (via ``svn add``).
+
+    ``overwrite``: If false, then don't every overwrite anything.
+
+    ``interactive``: If you are overwriting a file and interactive is
+    true, then ask before overwriting.
+
+    ``template_renderer``: This is a function for rendering templates
+    (if you don't want to use Cheetah or string.Template).  It should
+    have the signature ``template_renderer(content_as_string,
+    vars_as_dict, filename=filename)``.
+    """
+    # This allows you to use a leading +dot+ in filenames which would
+    # otherwise be skipped because leading dots make the file hidden:
+    vars.setdefault('dot', '.')
+    vars.setdefault('plus', '+')
     names = os.listdir(source)
     names.sort()
     pad = ' '*(indent*2)
@@ -56,9 +97,12 @@ def copy_dir(source, dest, vars, verbosity, simulate, indent=0,
         content = f.read()
         f.close()
         if sub_file:
-            content = substitute_content(content, vars, filename=full,
-                                         use_cheetah=use_cheetah,
-                                         template_renderer=template_renderer)
+            try:
+                content = substitute_content(content, vars, filename=full,
+                                             use_cheetah=use_cheetah,
+                                             template_renderer=template_renderer)
+            except SkipTemplate:
+                continue
         already_exists = os.path.exists(dest_full)
         if already_exists:
             f = open(dest_full, 'rb')
@@ -216,11 +260,12 @@ def substitute_filename(fn, vars):
 def substitute_content(content, vars, filename='<string>',
                        use_cheetah=False, template_renderer=None):
     global Cheetah
+    v = standard_vars.copy()
+    v.update(vars)
+    vars = v
     if template_renderer is not None:
         return template_renderer(content, vars, filename=filename)
     if not use_cheetah:
-        v = standard_vars.copy()
-        v.update(vars)
         tmpl = LaxTemplate(content)
         try:
             return tmpl.substitute(TypeMapper(v))
@@ -281,6 +326,15 @@ def test(conf, true_cond, false_cond=None):
     else:
         return false_cond
 
+def skip_template(condition=True):
+    """
+    Raise SkipTemplate, which causes copydir to skip the template
+    being processed.  If you pass in a condition, only raise if that
+    condition is true (allows you to use this with string.Template)
+    """
+    if condition:
+        raise SkipTemplate()
+
 def _add_except(exc, info):
     if not hasattr(exc, 'args') or exc.args is None:
         return
@@ -299,6 +353,8 @@ standard_vars = {
     'repr': repr,
     'str': str,
     'bool': bool,
+    'SkipTemplate': SkipTemplate,
+    'skip_template': skip_template,
     }
 
 class TypeMapper(dict):
@@ -307,12 +363,12 @@ class TypeMapper(dict):
         options = item.split('|')
         for op in options[:-1]:
             try:
-                value = eval_with_catch(op, dict(self))
+                value = eval_with_catch(op, dict(self.items()))
                 break
             except (NameError, KeyError):
                 pass
         else:
-            value = eval(options[-1], dict(self))
+            value = eval(options[-1], dict(self.items()))
         if value is None:
             return ''
         else:
