@@ -12,18 +12,24 @@ import errno
 import signal
 import sys
 import time
+try:
+    import subprocess
+except ImportError:
+    from paste.util import subprocess24 as subprocess
 from command import Command, BadCommand
 from paste.deploy import loadapp, loadserver
 import threading
 import atexit
+import logging
+import logging.config
+import ConfigParser
 
 MAXFD = 1024
 
 class ServeCommand(Command):
 
-    max_args = 2
     min_args = 0
-    usage = 'CONFIG_FILE [start|stop|restart|status]'
+    usage = 'CONFIG_FILE [start|stop|restart|status] [var=value]'
     takes_config_file = 1
     summary = "Serve the described application"
     description = """\
@@ -32,6 +38,9 @@ class ServeCommand(Command):
     
     If start/stop/restart is given, then --daemon is implied, and it will
     start (normal operation), stop (--stop-daemon), or do both.
+
+    You can also include variable assignments like 'http_port=8080'
+    and then use %(http_port)s in your config files.
     """
     
     # used by subclasses that configure apps and servers differently
@@ -70,13 +79,18 @@ class ServeCommand(Command):
                       dest='reload_interval',
                       default=1,
                       help="Seconds between checking files (low number can cause significant CPU usage)")
+    parser.add_option('--monitor-restart',
+                      dest='monitor_restart',
+                      action='store_true',
+                      help="Auto-restart server if it dies")
     parser.add_option('--status',
                       action='store_true',
                       dest='show_status',
                       help="Show the status of the (presumably daemonized) server")
 
+
     if hasattr(os, 'setuid'):
-        # I don't think these are availble on Windows
+        # I don't think these are available on Windows
         parser.add_option('--user',
                           dest='set_user',
                           metavar="USERNAME",
@@ -92,12 +106,14 @@ class ServeCommand(Command):
                       help='Stop a daemonized server (given a PID file, or default paster.pid file)')
 
 
-    _scheme_re = re.compile(r'^[a-zA-Z0-9_-]+:')
+    _scheme_re = re.compile(r'^[a-z][a-z]+:', re.I)
 
     default_verbosity = 1
 
     _reloader_environ_key = 'PYTHON_RELOADER_SHOULD_RUN'
+    _monitor_environ_key = 'PASTE_MONITOR_SHOULD_RUN'
 
+    possible_subcommands = ('start', 'stop', 'restart', 'status')
     def command(self):
         if self.options.stop_daemon:
             return self.stop_daemon()
@@ -113,17 +129,23 @@ class ServeCommand(Command):
             if not self.args:
                 raise BadCommand('You must give a config file')
             app_spec = self.args[0]
-            if len(self.args) > 1:
+            if (len(self.args) > 1
+                and self.args[1] in self.possible_subcommands):
                 cmd = self.args[1]
+                restvars = self.args[2:]
             else:
                 cmd = None
+                restvars = self.args[1:]
         else:
             app_spec = ""
-            if self.args:
+            if (self.args
+                and self.args[0] in self.possible_subcommands):
                 cmd = self.args[0]
+                restvars = self.args[1:]
             else:
                 cmd = None
-            
+                restvars = self.args[:]
+
         if self.options.reload:
             if os.environ.get(self._reloader_environ_key):
                 from paste import reloader
@@ -135,6 +157,10 @@ class ServeCommand(Command):
             else:
                 return self.restart_with_reloader()
 
+        if (self.options.monitor_restart
+            and not os.environ.get(self._monitor_environ_key)):
+            return self.restart_with_monitor()
+
         if cmd not in (None, 'start', 'stop', 'restart', 'status'):
             raise BadCommand(
                 'Error: must give start|stop|restart (not %s)' % cmd)
@@ -154,6 +180,7 @@ class ServeCommand(Command):
                 return result
 
         app_name = self.options.app_name
+        vars = self.parse_vars(restvars)
         if not self._scheme_re.search(app_spec):
             app_spec = 'config:' + app_spec
         server_name = self.options.server_name
@@ -165,6 +192,15 @@ class ServeCommand(Command):
             server_spec = app_spec
         base = os.getcwd()
 
+        # Ensure the log file is writeable
+        if self.options.log_file:
+            try:
+                writeable_log_file = open(self.options.log_file, 'a')
+            except IOError, ioe:
+                msg = 'Error: Unable to write to log file: %s' % ioe
+                raise BadCommand(msg)
+            writeable_log_file.close()
+
         if self.options.daemon:
             self.daemonize()
 
@@ -172,16 +208,27 @@ class ServeCommand(Command):
             self.record_pid(self.options.pid_file)
 
         if self.options.log_file:
-            stdout_log = LazyWriter(self.options.log_file)
+            stdout_log = LazyWriter(self.options.log_file, 'a')
             sys.stdout = stdout_log
             sys.stderr = stdout_log
-            # @@: Should we also redirect logging-based logs to this file?
-            # e.g., the root logger?
+            logging.basicConfig(stream=stdout_log)
+
+        log_fn = app_spec
+        if log_fn.startswith('config:'):
+            log_fn = app_spec[len('config:'):]
+        elif log_fn.startswith('egg:'):
+            log_fn = None
+        if log_fn:
+            log_fn = os.path.join(base, log_fn)
+            parser = ConfigParser.ConfigParser()
+            parser.read([log_fn])
+            if parser.has_section('loggers'):
+                logging.config.fileConfig(log_fn)
 
         server = self.loadserver(server_spec, name=server_name,
-                            relative_to=base)
+                                 relative_to=base, global_conf=vars)
         app = self.loadapp(app_spec, name=app_name,
-                      relative_to=base)
+                           relative_to=base, global_conf=vars)
 
         if self.verbose > 0:
             print 'Starting server in PID %i.' % os.getpid()
@@ -196,12 +243,15 @@ class ServeCommand(Command):
                 msg = ''
             print 'Exiting%s (-v to see traceback)' % msg
     
-    def loadserver(self, server_spec, name, relative_to):
-            return loadserver(server_spec, name=name,
-             relative_to=relative_to)
+    def loadserver(self, server_spec, name, relative_to, **kw):
+            return loadserver(
+                server_spec, name=name,
+                relative_to=relative_to, **kw)
     
-    def loadapp(self, app_spec, name, relative_to):
-            return loadapp(app_spec, name=name, relative_to=relative_to)
+    def loadapp(self, app_spec, name, relative_to, **kw):
+            return loadapp(
+                app_spec, name=name, relative_to=relative_to,
+                **kw)
 
     def daemonize(self):
         if self.verbose > 0:
@@ -302,23 +352,47 @@ class ServeCommand(Command):
         return 0
 
     def restart_with_reloader(self):
+        self.restart_with_monitor(reloader=True)
+
+    def restart_with_monitor(self, reloader=False):
         if self.verbose > 0:
-            print 'Starting subprocess with file monitor'
+            if reloader:
+                print 'Starting subprocess with file monitor'
+            else:
+                print 'Starting subprocess with monitor parent'
         while 1:
-            args = [sys.executable] + sys.argv
-            if sys.platform == "win32":
-                args = ['"%s"' % arg for arg in args]
+            args = [self.quote_first_command_arg(sys.executable)] + sys.argv
             new_environ = os.environ.copy()
-            new_environ[self._reloader_environ_key] = 'true'
+            if reloader:
+                new_environ[self._reloader_environ_key] = 'true'
+            else:
+                new_environ[self._monitor_environ_key] = 'true'
+            proc = None
             try:
-                exit_code = os.spawnve(os.P_WAIT, sys.executable,
-                                       args, new_environ)
-            except KeyboardInterrupt:
-                if self.verbose > 1:
-                    raise
-                return 1
-            if exit_code != 3:
-                return exit_code
+                try:
+                    _turn_sigterm_into_systemexit()
+                    proc = subprocess.Popen(args, env=new_environ)
+                    exit_code = proc.wait()
+                    proc = None
+                except KeyboardInterrupt:
+                    print '^C caught in monitor process'
+                    if self.verbose > 1:
+                        raise
+                    return 1
+            finally:
+                if (proc is not None
+                    and hasattr(os, 'kill')):
+                    import signal
+                    try:
+                        os.kill(proc.pid, signal.SIGTERM)
+                    except (OSError, IOError):
+                        pass
+                
+            if reloader:
+                # Reloader always exits with code 3; but if we are
+                # a monitor, any exit code will restart
+                if exit_code != 3:
+                    return exit_code
             if self.verbose > 0:
                 print '-'*20, 'Restarting', '-'*20
 
@@ -361,17 +435,23 @@ class ServeCommand(Command):
             
 class LazyWriter(object):
 
-    def __init__(self, filename):
+    """
+    File-like object that opens a file lazily when it is first written
+    to.
+    """
+
+    def __init__(self, filename, mode='w'):
         self.filename = filename
         self.fileobj = None
         self.lock = threading.Lock()
+        self.mode = mode
         
     def open(self):
         if self.fileobj is None:
             self.lock.acquire()
             try:
                 if self.fileobj is None:
-                    self.fileobj = open(self.filename, 'a')
+                    self.fileobj = open(self.filename, self.mode)
             finally:
                 self.lock.release()
         return self.fileobj
@@ -487,3 +567,16 @@ def _cleanup_ports(bound_addresses, maxtries=30, sleeptime=2):
         else:
             raise SystemExit('Timeout waiting for port.')
         sock.close()
+
+def _turn_sigterm_into_systemexit():
+    """
+    Attempts to turn a SIGTERM exception into a SystemExit exception.
+    """
+    try:
+        import signal
+    except ImportError:
+        return
+    def handle_term(signo, frame):
+        raise SystemExit
+    signal.signal(signal.SIGTERM, handle_term)
+    
